# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

from poaupdater import pbaapi as module_pbaapi, uLogging
from openapi import getVendorCompany
from pbadb import getStoreTemplatesDates
from utils import forbid_for_simulation_mode

import traceback

api = None

# Constants
API_SORT_NUMBER = 2


class PbaException(Exception):
    pass


class BMVersion:
    def __init__(self,
                 productVersion=None,
                 buildVersion=None,
                 installDate=None):

        self.productVersion = productVersion
        self.buildVersion = buildVersion
        self.installDate = installDate


class Store:
    def __init__(self,
                 storeId=None,
                 vendorId=None,
                 name=None,
                 companyName=None,
                 fullName=None,
                 shortName=None):

        self.storeId = storeId
        self.vendorId = vendorId
        self.name = name
        self.companyName = companyName
        self.fullName = fullName
        self.shortName = shortName


class StoreTemplate:
    def __init__(self,
                 templateId=None,
                 storeId=None,
                 description=None,
                 content=None):

        self.templateId = templateId
        self.storeId = storeId
        self.description = description
        self.content = content


class StoreTemplateHistory:
    def __init__(self,
                 templateId=None,
                 storeId=None,
                 date=None):

        self.templateId = templateId
        self.storeId = storeId
        self.date = date


def init():
    global api
    # Initialized BMAPI settings
    module_pbaapi.initFromEnv()
    # Retrieve BMAPI client
    api = module_pbaapi.PBAAPIRaw()


def getBMVersionList():
    """ Get versions history from PBA 
    :rtype: list of BMVersion
    """

    resp = api.BM.GetBMVersionList(4)
    versions = resp['Result'][0]
    result = []

    for version in versions:
        result.append(BMVersion(
            productVersion=version[0],
            buildVersion=version[1],
            installDate=version[3]
        ))
    return result


def getStoreList(storeId=None):
    """ Retrieve a list of active online stores
    :rtype: dict
    """

    resp = api.STORES.StorePackageListGet_API(API_SORT_NUMBER)
    packages = resp['Result'][0]
    result = {}

    for store in [store for store in packages if not storeId or store[0] == storeId]:
        if store[5] == 1:  # IsActive
            storeId = store[0]
            vendorId = store[1]
            name = store[2]
            companyName = getVendorCompany(vendorId)
            fullName = u"Online Store #{id} '{name}' (vendor={vendor}{additional_vendor_info})".format(
                id=storeId,
                name=name,
                vendor=vendorId,
                additional_vendor_info=(u", company='%s'" % companyName) if companyName else ''
            )
            shortName = u"Online Store #{id} '{name}' (vendor={vendor})".format(
                id=storeId,
                name=name,
                vendor=vendorId,
            )

            result[storeId] = Store(
                storeId=storeId,
                vendorId=vendorId,
                name=name,
                companyName=companyName,
                fullName=fullName,
                shortName=shortName
            )

    return result


def getStoreTemplateDetails(templateId, storeId):
    """ Retrieve details for selected template 
    :param string templateId:
    :param int storeId:
    :rtype: StoreTemplate
    """

    resp = api.STORES.StoreLayoutGet(templateId, storeId)
    template = resp['Result'][0]
    result = StoreTemplate(
        templateId=template[0],
        storeId=template[1],
        description=template[2],
        content=template[3]
    )

    return result


def getStoreTemplateList(storeId, isExtended=False):
    """ Retrieve a list of layout templates for selected online store
    :param int storeId:
    :param bool isExtended:
    :rtype: dict of StoreTemplate
    """

    templatesDates = getStoreTemplatesDates()
    if storeId not in templatesDates.keys():
        return {}

    result = {}
    failedTemplates = []

    for templateId in templatesDates[storeId].keys():
        if isExtended:
            try:
                result[templateId] = getStoreTemplateDetails(templateId, storeId)
            except:
                failedTemplates.append(templateId)
                uLogging.debug(traceback.format_exc())
        else:
            result[templateId] = StoreTemplate(
                templateId=templateId,
                storeId=storeId
            )

    if failedTemplates:
        raise PbaException('ERROR: Unable to retrieve templates for the online store #%d. This store will be skipped.\n'
                           'TEMPLATES:\n'
                           '    %s' % (storeId, '\n    '.join(failedTemplates)))

    return result


def getStoreTemplateHistoryList(templateId, storeId):
    """
    Retrieve history for selected template
    :param templateId:
    :param storeId:
    :rtype: list of StoreTemplateHistory
    """
    resp = api.STORES.StoreLayoutGetHistoryList(templateId, storeId, API_SORT_NUMBER)
    templates = resp['Result'][0]
    result = []

    for tpl in templates:
        result.append(StoreTemplateHistory(
            templateId=templateId,
            storeId=storeId,
            date=tpl[3]
        ))

    lastRevisionDates = getStoreTemplatesDates()
    result.append(StoreTemplateHistory(
        templateId=templateId,
        storeId=storeId,
        date=lastRevisionDates[storeId][templateId]
    ))

    return result


@forbid_for_simulation_mode
def updateStoreTemplate(templateId, storeId, description, content):
    """
    Update selected template
    :param templateId:
    :param storeId:
    :param description:
    :param content:
    """
    api.STORES.StoreLayoutUpdate(templateId, storeId, description, content)


@forbid_for_simulation_mode
def deleteStoreTemplate(templateId, storeId):
    """
    Delete selected template
    :param templateId:
    :param storeId:
    """
    api.STORES.StoreLayoutMoveToArc(templateId, storeId)


@forbid_for_simulation_mode
def synchronizeStore(storeId):
    api.STORES.StoreSyncConfSynchronize_API(storeId)
