# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

from poaupdater import uLogging

import traceback
from codecs import open
from os import path, mkdir, unlink, listdir
from sys import exit

from store_template import StoreTemplateManager
from snapshot import SnapshotManager
from version import VersionManager
from openapi import isPBAInstalled
from hashmap import HashedTemplatesStorage

from output import echo, ansi, bold, warn, indent
from output import ANSI_COLOR_FG_CYAN, ANSI_COLOR_FG_GREEN, ANSI_EFFECT_BOLD, ANSI_EFFECT_NEGATIVE

import pba
import constants
import utils
import re


DIFF_TEMPLATE_HAS_PATCH = 1
DIFF_TEMPLATE_NOT_EXISTS = 2
DIFF_TEMPLATE_IS_EQUAL = 3

STATUS_TEMPLATE_HAS_PATCH = 'PATCH'
STATUS_TEMPLATE_DELETE = 'DELETE'


class UpgradeException(Exception):
    def __init__(self, message, parent=None):
        if parent:
            message = message.format(message=parent.message)

        Exception.__init__(self, message)


class UpgradeManager:
    def __init__(self, dontSynchronizeStores=False, detailed=False, storeId=None, targetVersion=None):
        self.dontSynchronizeStores = dontSynchronizeStores
        self.outputMoreDetails = detailed
        self.preCheck()

        pba.init()

        echo('Loading data from Billing...')
        self.snapshotManager = SnapshotManager()
        self.storeTemplatesManager = StoreTemplateManager(storeId)
        self.versionManager = VersionManager(replaceCurrentVersionBy=targetVersion)
        self.hashedTemplatesStorage = HashedTemplatesStorage(replaceCurrentVersionBy=targetVersion)
        self.hashedTemplatesStorage.init()

        self.storeTemplates = self.storeTemplatesManager.getTemplates()
        self.stores = self.storeTemplatesManager.getStores()

        echo('All the required data were loaded from Billing.')
        echo()

    def preCheck(self):
        try:
            self.checkInstalledPBA()
            self.checkInstalledPackage('diffutils')
            self.checkInstalledPackage('patch')
        except Exception, e:
            echo('Pre-check error: %s' % unicode(str(e)))
            exit(255)

    @staticmethod
    def checkInstalledPBA():
        try:
            if not isPBAInstalled():
                raise UpgradeException(
                    'A required component, %s, is not installed yet. Nothing to do.' % constants.BILLING_PRODUCT_NAME
                )
        except Exception as e:
            if unicode(str(e)).startswith('Connection attempt to Open API refused.'):
                raise UpgradeException(
                    'You can run this utility only from the Odin Automation management node.'
                )
            raise

    @staticmethod
    def checkInstalledPackage(packageName):
        if utils.execCommand('rpm -qa | grep -w %s > /dev/null' % packageName):
            raise UpgradeException(
                "The required package '%s' is not installed. Try installing the required package with YUM: "
                "'yum install %s'" % (packageName, packageName)
            )

    def prepareDirectories(self, storeId=None, destinationDir=None):
        if not storeId:
            return

        dirs = self.getDirectories(storeId, destinationDir)
        path.exists(dirs['storeDir']) or mkdir(dirs['storeDir'])
        path.exists(dirs['templatesDir']) or mkdir(dirs['templatesDir'])
        path.exists(dirs['patchesDir']) or mkdir(dirs['patchesDir'])

    @staticmethod
    def getDirectories(storeId, destinationDir=None):
        if not destinationDir:
            destinationDir = constants.OUTPUT_DIR

        storeDir = path.join(destinationDir, 'store-%d' % storeId)

        return {
            'storeDir': storeDir,
            'templatesDir': path.join(storeDir, '.yours'),
            'patchesDir': path.join(storeDir, '.patches')
        }

    def cleanupDirectories(self, storeId=None, destinationDir=None):
        if storeId:
            dirs = self.getDirectories(storeId, destinationDir)
            utils.deleteDirectory(dirs['storeDir'])
            return
        if destinationDir and destinationDir != constants.OUTPUT_DIR:
            utils.deleteDirectory(destinationDir)

    @staticmethod
    def isUncompleted():
        return bool([d for d in listdir(constants.OUTPUT_DIR)
                     if d.startswith('store-') and path.isdir(path.join(constants.OUTPUT_DIR, d))])

    def checkPossibleConflicts(self, storeId=None, createDiffFromLastVersion=False, recursively=False):
        """
        Check possible conflicts in templates
        :param storeId:
        :param createDiffFromLastVersion:
        :param recursively: 
        :return: dict of {store:Store, hasErrors:<True/False>}
        :rtype: dict
        """
        destinationDir = constants.CHECK_DIR
        if not recursively:
            path.exists(destinationDir) or mkdir(destinationDir)

        if not storeId:
            # Do prepare for all stores recursively
            storeIds = [sid for sid, store in self.stores.items()]
            storeIds.sort()
            processedStores = {}
            try:
                for sid in storeIds:
                    processedStores.update(self.checkPossibleConflicts(
                        storeId=sid,
                        createDiffFromLastVersion=createDiffFromLastVersion,
                        recursively=True
                    ))
            finally:
                self.cleanupDirectories(destinationDir=destinationDir)

            return processedStores

        try:
            if storeId not in self.stores.keys():
                return {}

            self.storeTemplatesManager.loadTemplates(storeId)

            currentBMVersion = self.versionManager.getCurrentVersion().buildVersion

            # Prepare directories
            self.prepareDirectories(storeId, destinationDir)

            # Generate contents to check
            templatesHavingPatches = self.prepareTemplates(
                storeId=storeId,
                createDiffFromLastVersion=createDiffFromLastVersion,
                destinationDir=destinationDir,
            )

            if not templatesHavingPatches:
                return {storeId: {
                    'store': self.stores[storeId],
                    'hasErrors': False
                }}
            else:
                if self.outputMoreDetails:
                    echo(ansi('Processing the %s...' % self.stores[storeId].fullName, [ANSI_COLOR_FG_CYAN]))
                    echo()
                    echo(bold('Layout templates used by the {store_name} with ID #{store_id} differ from the current '
                              'version of {billing_name} {template_version}:'.format(
                                store_name=constants.ONLINE_STORE_PRODUCT_NAME,
                                store_id=storeId,
                                billing_name=constants.BILLING_PRODUCT_NAME,
                                template_version=currentBMVersion)))
                    indent(4)
                    [echo(template['templateId']) for template in templatesHavingPatches]
                    indent(0)

                    echo()
                    echo(bold('We recommend that you apply the changes in layout templates to your customizations.'))
                    echo('Use the following command to start preparing layout templates:')
                    echo('# ./{program} {command}'.format(
                        program=constants.PROGRAM_NAME,
                        command=constants.PROGRAM_COMMAND_PREPARE,
                        store_id=storeId
                    ))
                    echo()
        finally:
            self.cleanupDirectories(storeId, destinationDir)
            if not recursively:
                self.cleanupDirectories(destinationDir=destinationDir)

        return {storeId: {
            'store': self.stores[storeId],
            'hasErrors': True
        }}

    def upgradeTemplates(self, storeId=None, createDiffFromLastVersion=False):
        """
        Upgrades templates
        :param storeId:
        :param createDiffFromLastVersion:
        :return: dict of {store:Store, hasErrors:<True/False>}
        :rtype: dict
        """
        if not storeId:
            # Do prepare for all stores recursively
            storeIds = [sid for sid, store in self.stores.items()]
            storeIds.sort()
            processedStores = {}
            for sid in storeIds:
                processedStores.update(self.upgradeTemplates(
                    storeId=sid,
                    createDiffFromLastVersion=createDiffFromLastVersion
                ))
            return processedStores

        if storeId not in self.stores.keys():
            return {}

        self.storeTemplatesManager.loadTemplates(storeId)

        # Prepare directories
        self.prepareDirectories(storeId)

        dirs = self.getDirectories(storeId)

        currentBMVersion = self.versionManager.getCurrentVersion().buildVersion
        storeDir = dirs['storeDir']

        # Generate contents for merge
        templatesHavingActions = self.prepareTemplates(
            storeId=storeId,
            createDiffFromLastVersion=createDiffFromLastVersion
        )

        # Copy prepared customization to backup
        try:
            backupStorePath = path.join(constants.BACKUP_DIR,'store-%d' % storeId)
            path.exists(backupStorePath) or utils.makeDirectory(backupStorePath)
            for backupTemplate in templatesHavingActions:
                filePath = backupTemplate['templateFilePath']
                utils.copyFile(filePath, path.join(backupStorePath, '%s.tpl' % backupTemplate['templateId']), True)
        except Exception as e:
            uLogging.debug(traceback.format_exc())
            raise UpgradeException('Can not create backup due to the error: {message}', e)

        templatesWithConflicts = None
        if not templatesHavingActions:
            utils.deleteDirectory(dirs['storeDir'])
        else:
            self.createStoreDescriptionFile(storeId)

            echo(ansi('=== Processing the %s...' % self.stores[storeId].fullName, [ANSI_COLOR_FG_CYAN]))
            echo()
            indent(4)
            echo(bold('Layout templates used by the {store_name} with ID #{store_id} differ from the current '
                      'version of {billing_name} {template_version}:'.format(
                        store_name=constants.ONLINE_STORE_PRODUCT_NAME,
                        store_id=storeId,
                        billing_name=constants.BILLING_PRODUCT_NAME,
                        template_version=currentBMVersion)))

            indent(8)
            maxTemplateNameLength = max([len(tpl['templateId']) for tpl in templatesHavingActions])

            [echo('{name:<{width}} last modified in oa-{version}'.format(
                    name=template['templateId'],
                    width=maxTemplateNameLength,
                    version=template['firstRevisionBuildVersion']
                )) for template in templatesHavingActions if template['status'] == STATUS_TEMPLATE_HAS_PATCH]

            echo()

            templatesToBeDeleted = [t for t in templatesHavingActions if t['status'] == STATUS_TEMPLATE_DELETE]
            if templatesToBeDeleted:
                indent(4)
                echo(bold('The following layout templates do not contain any changes and will be removed from the '
                          'billing:'))

                indent(8)
                [echo('{name:<{width}} last modified in oa-{version}'.format(
                        name=template['templateId'],
                        width=maxTemplateNameLength,
                        version=template['firstRevisionBuildVersion']
                    )) for template in templatesToBeDeleted]

                echo()

            indent(4)
            echo('The location of the patched layout templates:')
            echo(path.realpath(storeDir))
            echo()

            echo('Trying to correct templates automatically...')

            templatesWithConflicts = self.applyPatches(storeId)[storeId]

            if templatesWithConflicts:
                echo('Some of the layout templates were not corrected.')
                echo()
                echo(bold('You need to correct the following layout templates manually:'))
                indent(4)
                [echo(problemTemplateId) for problemTemplateId in templatesWithConflicts]
            else:
                echo('All layout templates were prepared.')

            indent(0)
            echo()

        return {storeId: {
            'store': self.stores[storeId],
            'hasErrors': bool(templatesWithConflicts),
            'templates': templatesWithConflicts
        }}

    def prepareTemplates(self, storeId, destinationDir=None, createDiffFromLastVersion=False):
        """
        Prepare templates files
        :param int storeId: 
        :param str destinationDir: 
        :param bool createDiffFromLastVersion: 
        :return: A list of templates that has a patch
        :rtype: list of dict
        """

        if storeId not in self.stores.keys():
            return []

        self.storeTemplatesManager.loadTemplates(storeId)

        # Prepare directories
        self.prepareDirectories(storeId, destinationDir)
        dirs = self.getDirectories(storeId, destinationDir)

        currentBMVersion = self.versionManager.getCurrentVersion().buildVersion
        storeTemplatesDir = dirs['templatesDir']
        patchesDir = dirs['patchesDir']

        # Generate contents for merge
        templatesHavingActions = []
        for storeTemplate in [template for templateId, template in self.storeTemplates[storeId].items()]:
            templateId = storeTemplate.templateId
            templateFileName = '%s.tpl' % templateId
            templateFilePath = path.join(storeTemplatesDir, templateFileName)

            lastRevisionDate = self.getLastRevisionDate(templateId, storeId)
            if lastRevisionDate is None:
                continue

            lastRevisionVersion = self.versionManager.getVersionForDate(lastRevisionDate)
            if lastRevisionVersion.buildVersion == self.versionManager.getCurrentVersion().buildVersion:
                # Template is modified in latest build
                # Nothing to do
                continue

            startSnapshotTemplates = self.snapshotManager.getTemplates(
                BMVersion=lastRevisionVersion.buildVersion,
                templateId=templateId
            )
            lastSnapshotTemplates = self.snapshotManager.getTemplates(
                BMVersion=currentBMVersion,
                templateId=templateId
            )

            if templateId not in startSnapshotTemplates or templateId not in lastSnapshotTemplates:
                continue

            startSnapshotTemplate = startSnapshotTemplates[templateId]
            lastSnapshotTemplate = lastSnapshotTemplates[templateId]

            # Write templates retrieved from store
            f = open(templateFilePath, 'w', 'utf-8')
            try:
                uLogging.debug('Creating a file: %s' % templateFilePath)
                f.write(storeTemplate.content)
            except:
                uLogging.debug('Error:')
                uLogging.debug(traceback.format_exc())
            finally:
                f.close()

            # Make diff file between current and prior versions of snapshot template
            fromVersion = lastSnapshotTemplate.priorVersion \
                if createDiffFromLastVersion else startSnapshotTemplate.version

            makePatchResult = self.makePatch(
                storeId,
                fromVersion,
                templateId,
                patchesDir,
                destinationDir=destinationDir
            )

            if DIFF_TEMPLATE_HAS_PATCH == makePatchResult:
                templatesHavingActions.append({
                    'templateFilePath': templateFilePath,
                    'templateId': templateId,
                    'status': STATUS_TEMPLATE_HAS_PATCH,
                    'firstRevisionBuildVersion': lastRevisionVersion.buildVersion
                })
            elif DIFF_TEMPLATE_IS_EQUAL == makePatchResult:
                templatesHavingActions.append({
                    'templateFilePath': templateFilePath,
                    'templateId': templateId,
                    'status': STATUS_TEMPLATE_DELETE,
                    'firstRevisionBuildVersion': lastRevisionVersion.buildVersion
                })
                path.exists(templateFilePath) and utils.execCommand('cp {source} {destination} > /dev/null'.format(
                    source=templateFilePath,
                    destination=path.join(dirs['storeDir'], '%s.delete' % path.basename(templateFilePath))
                ))

        return templatesHavingActions

    def applyPatches(self, storeId=None):
        """
        :param storeId:
        :return:
        :rtype: dict
        """

        problemTemplates = {}

        if not storeId:
            # Apply patches for all stores recursively
            storeIds = [sid for sid, store in self.stores.items()]
            storeIds.sort()

            for sid in storeIds:
                problemTemplates[sid] = self.applyPatches(storeId=sid)[sid]
            return problemTemplates

        if storeId not in self.stores.keys():
            return {}

        dirs = self.getDirectories(storeId)
        patches = self.getPatches(storeId)

        toVersion = self.snapshotManager.getSnapshotVersion(
            self.versionManager.getCurrentVersion().buildVersion
        )
        toVersionPath = self.snapshotManager.getVersionPath(toVersion)

        problemTemplates[storeId] = []

        if not patches and not self.getTemplatesToBeDeleted(storeId):
            self.cleanupDirectories(storeId)
            return problemTemplates

        for patchFileName in patches:
            templateId = patchFileName.replace('.diff', '')
            templateFileName = patchFileName.replace('.diff', '.tpl')
            toVersionFilePath = path.join(toVersionPath, templateFileName)
            templateFilePath = path.join(dirs['storeDir'], templateFileName)
            patchFilePath = path.join(dirs['patchesDir'], patchFileName)

            originalFilePath = path.realpath('%s.orig' % templateFilePath)
            releaseFilePath = path.realpath('%s.latest-release' % templateFilePath)
            yoursFilePath = path.join(dirs['storeDir'], '%s.tpl.yours' % templateId)
            customizedFilePath = path.join(dirs['templatesDir'], templateFileName)

            utils.copyFile(path.realpath(toVersionFilePath), path.realpath(templateFilePath))

            customizedFileDigest = self.hashedTemplatesStorage.getFileDigest(customizedFilePath)
            resolvedPath = self.hashedTemplatesStorage.getResolvedTemplatePath(customizedFileDigest, templateId)
            if resolvedPath:
                path.exists(templateFilePath) and unlink(templateFilePath)

                # Current template is already resolved with another store
                utils.copyFile(resolvedPath, templateFilePath)

                [path.exists(_) and unlink(_) for _ in [originalFilePath, releaseFilePath, yoursFilePath]]

                continue

            result = utils.execCommand('patch -p0 --merge --ignore-whitespace {patched} {patch} > /dev/null'.format(
                patched=templateFilePath,
                patch=patchFilePath
            ))

            if result:
                # Patch is rejected

                # Rename new template delivered with new release
                path.exists(originalFilePath) and utils.execCommand('mv {source} {destination} > /dev/null'.format(
                    source=originalFilePath,
                    destination=releaseFilePath
                ))
                # Copy customized template without patches
                utils.copyFile(customizedFilePath, yoursFilePath)

                # Try to resolve templates conflict by hash storage
                # self.hashedTemplatesStorage.putTemplate(customizedFilePath)

                problemTemplates[storeId].append(templateId)
            else:
                # Patch is applied successfully
                # Cleanup directory
                # Remove copy of original file
                path.exists(originalFilePath) and unlink(originalFilePath)

        utils.deleteDirectory(dirs['templatesDir'])

        return problemTemplates

    def importPatchedTemplatesToStore(self, storeId=None):
        """
        :param int storeId:
        :return: dict of {store:Store, hasErrors:<True/False>}
        :rtype: dict
        """
        if not storeId:
            # Apply export patched templates for all stores recursively
            storeIds = self.stores.keys()
            storeIds.sort()
            processedStores = {}
            for sid in storeIds:
                processedStores.update(self.importPatchedTemplatesToStore(sid))
            return processedStores

        if storeId not in self.stores.keys():
            return {}

        self.storeTemplatesManager.loadTemplates(storeId)

        dirs = self.getDirectories(storeId)
        patches = self.getPatches(storeId)

        if not patches:
            return {}

        echo('### Importing the %s...' % self.stores[storeId].fullName)

        if storeId in self.detectConflicts(storeId).keys():
            return {storeId: {
                'store': self.stores[storeId],
                'hasErrors': True,
                'hashedTemplatesPaths': None
            }}

        oneMoreImported = False
        outputLine = '{name:<{width}} [{status}]'
        maxTemplateNameLength = max([len(name.replace('.diff', '')) for name in patches])
        hashedFilePaths = []

        indent(4)
        for patchFileName in patches:
            templateId = patchFileName.replace('.diff', '')

            if templateId not in self.storeTemplates[storeId]:
                continue

            templateFilePath = path.join(dirs['storeDir'], patchFileName.replace('.diff', '.tpl'))
            yoursTemplateFilePath = path.join(dirs['storeDir'], patchFileName.replace('.diff', '.tpl.yours'))

            if not path.exists(templateFilePath):
                continue

            if not oneMoreImported:
                oneMoreImported = True
                if self.outputMoreDetails:
                    echo(ansi('Templates', [ANSI_EFFECT_NEGATIVE]))

            f = open(templateFilePath, 'r', 'utf-8')
            try:
                content = f.read()
            finally:
                f.close()

            currentVersion = self.versionManager.getCurrentVersion().buildVersion

            oldTemplate = self.storeTemplates[storeId][templateId]
            description = u'Automatically updated to the version oa-%s\n%s' % (currentVersion, oldTemplate.description)

            if path.exists(yoursTemplateFilePath):
                # Move template to hash
                templateDigest = self.hashedTemplatesStorage.getFileDigest(yoursTemplateFilePath)
                hashedTemplate = self.hashedTemplatesStorage.putResolvedTemplate(templateDigest, templateFilePath)

                if hashedTemplate not in hashedFilePaths:
                    hashedFilePaths.append(hashedTemplate)

            try:
                pba.updateStoreTemplate(templateId, storeId, {'en': description}, content)
                unlink(templateFilePath)
                if self.outputMoreDetails:
                    echo(outputLine.format(
                        name=templateId,
                        status=ansi('imported', [ANSI_COLOR_FG_GREEN, ANSI_EFFECT_BOLD]),
                        width=maxTemplateNameLength
                    ))
            except Exception as e:
                if self.outputMoreDetails:
                    echo(outputLine.format(
                        name=templateId,
                        status=warn('failed with error: %s' % unicode(str(e))),
                        width=maxTemplateNameLength
                    ))
                uLogging.debug(e)

        for deleteTemplateFile in self.getTemplatesToBeDeleted(storeId):
            templateFilePath = path.join(dirs['storeDir'], deleteTemplateFile)
            templateId = deleteTemplateFile.replace('.tpl.delete', '')
            try:
                pba.deleteStoreTemplate(templateId, storeId)
                unlink(templateFilePath)
                if self.outputMoreDetails:
                    echo(outputLine.format(
                        name=templateId,
                        status=ansi('deleted', [ANSI_COLOR_FG_GREEN, ANSI_EFFECT_BOLD]),
                        width=maxTemplateNameLength
                    ))
            except Exception as e:
                if self.outputMoreDetails:
                    echo(outputLine.format(
                        name=templateId,
                        status=warn('failed with error: %s' % unicode(str(e))),
                        width=maxTemplateNameLength
                    ))
                uLogging.debug(e)

        indent(0)

        self.cleanupDirectories(storeId)
        if oneMoreImported:
            indent(4)
            if self.outputMoreDetails:
                echo()
                echo('All layout templates were imported to %s.' % constants.BILLING_PRODUCT_NAME)
                echo()
            if not self.dontSynchronizeStores:
                try:
                    if self.outputMoreDetails:
                        echo('Trying to synchronize %s...' % constants.ONLINE_STORE_PRODUCT_NAME)
                    pba.synchronizeStore(storeId)
                    if self.outputMoreDetails:
                        echo('The %s was synchronized.' % constants.ONLINE_STORE_PRODUCT_NAME)
                        echo()
                except Exception:
                    uLogging.debug(traceback.format_exc())

                    if self.outputMoreDetails:
                        echo(warn('The {store_name} was not synchronized because errors occurred.'.format(
                            store_name=constants.ONLINE_STORE_PRODUCT_NAME
                        )))
                        echo('To see more details, refer to %s' % constants.LOG_FILE)
                        echo()
            indent(0)

        return {storeId: {
            'store': self.stores[storeId],
            'hasErrors': False,
            'hashedTemplatesPaths': (hashedFilePaths or None)
        }}

    def getNotResolvedTemplatesWithDigests(self):
        paths = {}
        for storeId, _ in self.stores.items():
            storePaths = self.getFilesList(self.getDirectories(storeId)['storeDir'], 'tpl.yours', True)
            for filePath in storePaths:
                pathWithoutExtension = filePath.replace('.tpl.yours', '')

                digest = self.hashedTemplatesStorage.getFileDigest(filePath)
                if digest not in paths.keys():
                    paths[digest] = []

                paths[digest].append({
                    'storeId': storeId,
                    'templateId': path.basename(pathWithoutExtension),
                    'path': '%s.tpl' % pathWithoutExtension
                })
        return paths

    def applyHashedTemplates(self, hashedTemplatesPaths):
        result = {}
        for digest, templates in self.getNotResolvedTemplatesWithDigests().items():
            for options in templates:
                if not path.exists(options['path']):
                    continue

                foundName = self.hashedTemplatesStorage.generateFilePath(digest, templateId=options['templateId'])
                if foundName not in hashedTemplatesPaths:
                    continue

                utils.copyFile(foundName, options['path'])

                yoursFile = '%s.yours' % options['path']
                path.exists(yoursFile) and unlink(yoursFile)

                releaseFile = '%s.latest-release' % options['path']
                path.exists(releaseFile) and unlink(releaseFile)

                result[options['storeId']] = {
                    'store': self.stores[options['storeId']],
                    'hasErrors': False,
                }

        return result

    @staticmethod
    def getFilesList(directory, extension, fullPath=False):
        if not path.exists(directory):
            return []

        def prepareFileName(fileName):
            if not fullPath:
                return fileName
            return path.join(directory, fileName)

        return [prepareFileName(f) for f in listdir(directory) if f.endswith('.%s' % extension)]

    def getPatches(self, storeId=None):
        if not storeId:
            return []

        return self.getFilesList(self.getDirectories(storeId)['patchesDir'], 'diff')

    def getTemplatesToBeDeleted(self, storeId):
        return [f for f in self.getFilesList(self.getDirectories(storeId)['storeDir'], 'delete')]

    def detectConflicts(self, storeId=None):
        problemStores = {}
        for sid in self.stores.keys():
            if storeId is not None and storeId != sid:
                continue

            if not self.getTemplatesWithConflicts(sid):
                continue

            problemStores[sid] = self.stores[sid]

        return problemStores

    def getTemplatesWithConflicts(self, storeId):
        storeDir = self.getDirectories(storeId)['storeDir']
        if not path.exists(storeDir):
            return []

        grepOutput = utils.readCommand("grep -zl '<<<<<<<.*=======.*>>>>>>>' %s/*.tpl" % storeDir).strip()

        if not grepOutput:
            return []

        return re.split('\n', grepOutput)

    @staticmethod
    def getLastRevisionDate(templateId, storeId):
        """
        Retrieve creation date for last revision of customized template
        :param templateId:
        :param storeId:
        :rtype: int
        """
        history = pba.getStoreTemplateHistoryList(templateId, storeId)
        if not history:
            return None

        return history[-1].date

    def makePatch(self, storeId=None, fromVersion=None, templateId=None, patchesDir=None, destinationDir=None):
        """
        Make patches based on difference between old and new templates
        :param storeId:
        :param fromVersion:
        :param templateId:
        :param patchesDir:
        :param destinationDir:
        :return: True in case when old and new templates are differ
        :rtype: bool
        """

        if storeId is None or templateId is None or fromVersion is None or patchesDir is None:
            raise AssertionError('One of the required parameters is absent.')

        oldDir = self.snapshotManager.getVersionPath(fromVersion)
        newDir = self.getDirectories(storeId, destinationDir)['templatesDir']

        patchFilePath = path.join(patchesDir, '%s.%s' % (templateId, 'diff'))

        templateFileName = '%s.%s' % (templateId, 'tpl')
        oldFilePath = path.join(oldDir, templateFileName)
        newFilePath = path.join(newDir, templateFileName)

        if not path.isfile(oldFilePath) or not path.isfile(newFilePath):
            return DIFF_TEMPLATE_NOT_EXISTS

        result = utils.execCommand('diff '
                                   '--strip-trailing-cr '
                                   '--ignore-blank-lines '
                                   '--ignore-tab-expansion '
                                   '--ignore-space-change '
                                   '-C 3 -w -s %s %s > %s' % (oldFilePath, newFilePath, patchFilePath))

        if not result and path.isfile(patchFilePath):
            unlink(patchFilePath)
            return DIFF_TEMPLATE_IS_EQUAL

        return DIFF_TEMPLATE_HAS_PATCH

    def createStoreDescriptionFile(self, storeId):
        storeDir = self.getDirectories(storeId)['storeDir']
        descriptionFile = file(path.join(storeDir, '.store-description.txt'), 'w')
        try:
            store = self.stores[storeId]
            descriptionFile.write(u'\n'.join((
                u'Store ID:     %d' % storeId,
                u'Vendor ID:    %s' % store.vendorId,
                u'Name:         %s' % store.name,
                u'Company Name: %s' % store.companyName)).encode('utf-8'))
        finally:
            descriptionFile.close()

